/**
 * Copyright (C) 2000  Johan Redestig, Anders Palsson
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
/*
 * $Author: apl $
 * $Revision: 1.1.1.1 $
 * $Date: 2001/01/05 16:28:14 $
 */

import java.util.*;
import java.io.*;
import java.net.*;
import java.lang.reflect.*;

import javax.servlet.*;
import javax.servlet.http.*;

import org.log4j.Category;
import org.log4j.PropertyConfigurator;
import org.log4j.Layout;
import org.log4j.PatternLayout;
import org.log4j.FileAppender;

import org.codemode.bishop.View;
import org.codemode.bishop.Model;
import org.codemode.bishop.Scope;
import org.codemode.bishop.BishopDefinitions;

public class Bishop extends HttpServlet
{
    Category cat = Category.getInstance(Bishop.class.getName());
      /**
       * Name of the variable that is supplied by the client (in the request)
       * that defines which bishop request to execute.
       */
      public static final String BISHOP_REQUEST = "root.xml";
      
      /**
       */
      public void init(ServletConfig servletConfig) throws ServletException
      {
         super.init(servletConfig);
         PropertyConfigurator.configure();
      }
      
      /**
       */
      public void doGet(HttpServletRequest request, HttpServletResponse response)
         throws ServletException, IOException
      {
         doPost(request, response);
      }
      
      /**
       */
      public void doPost(HttpServletRequest request, HttpServletResponse response)
         throws ServletException, IOException
      {
         PrintWriter out = response.getWriter();
         Model model = new Model(request.getSession(true));

         out.print(invokeBishop(BISHOP_REQUEST,
                                createLocalVariables(request, response),
                                model));
         out.flush();
      }

      /**
       * Copy all variables from the request and session to a Map. 
       */
      protected Scope createLocalVariables(HttpServletRequest request, HttpServletResponse response)
      {
         Scope locals = new Scope(request);
         locals.putObject(BishopDefinitions.REQUEST, request);
         locals.putObject(BishopDefinitions.RESPONSE, response);
         return locals;
      }

      /**
       *
       */
      public String invokeBishop(String bishopName, Scope locals, Model model)
      {
         String result = null;
         View view;
         org.codemode.bishop.Bishop b;
         
         try
         {
            cat.debug("invoke: " + bishopName);
            view = (View)model.getObject(BishopDefinitions.VIEW);
            if (view == null)
               view = new View();

            b = new org.codemode.bishop.Bishop(bishopName);
            b.invoke(locals, view, model);
            view.put("Response", locals.getObject(BishopDefinitions.RESPONSE));
            result = view.evaluate();
            model.setObject(BishopDefinitions.VIEW, view);
         }
         catch (Throwable e)
         {
            cat.error("BishopServlet.invokeBishop error: ", e);
            e.printStackTrace();
         }
         cat.debug("invoke return: " + result);         
         return result;
      }
            
}

