/**
 * Copyright (C) 2000  Johan Redestig, Anders Palsson
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.codemode.bishop;

import java.util.Map;
import java.util.Map;
import java.io.DataInputStream;
import java.io.InputStream;

import javax.servlet.*;
import javax.servlet.http.*;

import org.xml.sax.SAXException;

import org.log4j.Category;

public class Bishop extends BishopEntity
{
      protected static final String NAME = "name";
      Category cat = Category.getInstance(Bishop.class.getName());

      public Bishop()
      {
         super();
      }
           
      public Bishop(String s)
      {
         super();
         setName(s);
      }
      
      /**
       * 
       */
      public void invoke(Scope locals, org.codemode.bishop.View view, Model model) throws Throwable
      {
         cat.debug("invoke");

         try
         {            
            if (getName() == null)
            {
               invokeBishop(locals, view, model);
            }
            else
            {
               invokeImport(locals, view, model);
            }
         }
         catch(InvalidBishopException e)
         {
            reportError(e.getInner(), this);
         }
         catch(BishopException e)
         {
            reportError(e.getInner() == null ? e : e.getInner(), e.getBishop());
         }
         catch(SAXException e)
         {
            reportError(e.getException() == null ? e : e.getException(), this);
         }
         catch(Throwable e)
         {
            reportError(e, this);
         }
      }

      /**
       * Invoke a <Import> statement
       */
      protected void invokeImport(Scope locals, org.codemode.bishop.View view, Model model) throws Throwable
      {
         cat.debug("invokeImport");         
         // if object doesn't exist in locals or session. try to get it as a file.

         String name = (String)attributes.get(NAME);
         Bishop bishop;
         String bishopName;
         java.lang.Object o = locals.getObject(name);

         if (o == null)
         {
            bishopName = name;
         }
         else
         {
            bishopName = (String)o;
         }

         bishop = importBishop(bishopName);
         cat.debug("importedBishop: " + bishop);
         bishop.invoke(locals, view, model);
         setName(null);        
      }

      /**
       *
       */
      protected void invokeBishop(Scope locals, org.codemode.bishop.View view, Model model) throws Throwable
      {
         cat.debug("invokeBishop: " + (String)attributes.get(NAME));
         invokeAllChildren(locals, view, model);
      }
      
      /**
       * The the name.
       */
      public void setName(String n)
      {
         super.setAttribute(NAME, n);
      }

      /**
       *
       */
      public String getName()
      {
         return (String)getAttribute(NAME);
      }
      
      /**
       *
       */
      protected Bishop importBishop(String bishopName) throws Exception
      {
         String request;
         Bishop importEntity = new Bishop();

         if (bishopName != null)
         {
            InputStream is = getClass().getResourceAsStream("/" + bishopName);
            if(is == null)
               throw new BishopException(this, "No such bishop: " + bishopName);
            DataInputStream in = new DataInputStream(is);
            byte b[] = new byte[in.available()];
            
            in.readFully (b);
            in.close();
            request = new String(b);
            
            importEntity = new BishopParser().parse(request);
         }

         return importEntity;
      }

      protected void reportError(Throwable e, BishopEntity b)
      {
         cat.error(e.getMessage());
         cat.error("Bishop stack: " + b.getStackTrace());
         cat.error("Internal stack", e);
      }
}
