/**
 * Copyright (C) 2000  Johan Redestig, Anders Palsson
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.codemode.bishop;

import javax.servlet.*;
import javax.servlet.http.*;

import org.xml.sax.AttributeList;
import org.xml.sax.helpers.AttributeListImpl;
import java.util.Vector;
import java.util.Enumeration;
import java.util.Map;
import java.util.HashMap;

public abstract class BishopEntity
{
      protected Vector children;
      protected HashMap attributes = new HashMap();
      protected String bishopTagName;
      protected BishopEntity parent = null;

      /**
       *
       */
      public BishopEntity()
      {
      }

      public void addChild(BishopEntity child)
      {
         if (children == null)
         {
            children = new Vector();
         }

         children.add(child);            
      }

      public Vector getChildren()
      {
         return children;
      }

      public void setParent(BishopEntity parent)
      {
         parent = parent;
      }

      public void setAttributes(AttributeList attributes)
      {
         this.attributes = new HashMap();

         for (int index = 0; index < attributes.getLength(); index++)
         {
            this.attributes.put(attributes.getName(index), attributes.getValue(index));
         }
      }

      public String getAttribute(String name)
      {
         return (String)attributes.get(name);
      }

      public void setAttribute(String name, java.lang.Object value)
      {
         attributes.put(name, value);
      }

      public String getBishopTagName()
      {
         return bishopTagName;
      }

      public void setBishopTagName(String tag)
      {
         bishopTagName = tag;
      }

      /**
       *
       */
      public abstract void invoke(Scope locals, View view, Model model) throws Throwable;


      /**
       * Invokes all children to this BishopEntity. A new local scope is created for
       * the 
       */
      protected void invokeAllChildren(Scope locals, View view, Model model) throws Throwable
      {
         Enumeration e;
         Scope innerScope;
         
         if (children != null)
         {
            e = children.elements();

            innerScope = locals.push();
            while (e.hasMoreElements())
            {
               BishopEntity bishop = (BishopEntity)e.nextElement();
               bishop.invoke(innerScope, view, model);
            }
         }
      }

      /**
       * Returns the object with the specifed name from locals, view or the session. Or
       * <code>null</code> if the object can't be found.
       */
      public String getStackTrace()
      {
         String result = "\n\t" + bishopTagName + " " + attributes.toString();
         if(parent != null)
            result += parent.getStackTrace();
         return result;
      }
      
      /**
       *
       */
      public String toString()
      {
         String s;
         String values = "";
         String names[] = new String[attributes.size()];
         
         names = (String[])attributes.keySet().toArray(names);

         for (int index = 0; index < attributes.size(); index++)
         {
            values += names[index] + "=" + attributes.get(names[index]) + ", "; 
         }
         
         if (children == null)
         {
            s = bishopTagName + "=[#children: 0, attribs:" + values + "]";
         }
         else
         {
            s = bishopTagName + "=[#children:" + children.size() + ", attribs:" + values + "]";
         }

         return s;
      }
}
 
