/**
 * Copyright (C) 2000  Johan Redestig, Anders Palsson
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.codemode.bishop;
import javax.servlet.http.*;
import java.util.HashMap;


/**
 * Represents the blocks in Bishop. A scope contains local variables for the current
 * scope (block) and a reference to its parent scope.
 */
public class Scope
{
      protected HashMap local;
      protected Scope parent;
      protected HttpServletRequest request;

      public Scope()
      {
         this(null);
      }

      public Scope(HttpServletRequest request)
      {
         this.request = request;
         local = new HashMap(5);
      }

      public Scope push()
      {
         Scope s = new Scope(request);
         s.parent = this;
         return s;
      }
      
      /**
       * Associates the specified value with the specified key in this scope.
       */
      public void putObject(java.lang.Object key, java.lang.Object value)
      {
         Scope s = this;
         while(s != null && !s.local.containsKey(key))
         {
            s = s.parent;
         }
         s = s == null ? this : s;
         s.local.put(key, value);
      }

      public java.lang.Object getObject(org.codemode.bishop.entity.In key) throws Exception
      {
         java.lang.Object value = null;
         java.lang.String type = key.getType();
         java.lang.String name = key.getName();
         Class c = null;

         if(type == null)
            c = null;
         else if(type.equals("int"))
            c = Integer.TYPE;
         else if(type.equals("boolean"))
            c = Boolean.TYPE;         
         else if(type.equals("byte"))
            c = Byte.TYPE;
         else if(type.equals("char"))
            c = Character.TYPE;
         else if(type.equals("double"))
            c = Double.TYPE;
         else if(type.equals("float"))
            c = Float.TYPE;
         else if(type.equals("long"))
            c = Long.TYPE;
         else if(type.equals("short"))
            c = Short.TYPE;
         else
            c = Class.forName(type);
         
         if(c != null && c.isArray())
            value = request.getParameterValues(name);
         else
            value = getObject(name);
         
         value = value == null ? key.getDefaultValue() : value;

         if(value == null)
            throw new BishopException(null, null, "No object or default-value");
         
         if(type == null || type.equals(value.getClass().getName()))
            return value;

         java.lang.Object o = null;
         if(c.isArray())
         {
            java.lang.Class d = c.getComponentType();
            java.lang.Object[] values = (java.lang.Object[])value;
            o = java.lang.reflect.Array.newInstance(d, values.length);
            for(int i = 0; i < values.length; i++)
            {
               java.lang.Class[] sig = new Class[] {String.class};
               java.lang.String[] arg = new String[] {(String)values[i]};               
               java.lang.Object tmp = c.getConstructor(sig).newInstance(arg);
               java.lang.reflect.Array.set(o, i, tmp);
            }
         }
         else
         {         
            // if it is not a primitive type then it must be of a class
            // that has a String contstructor (how would be otherwise be able
            // to marshall the String into an Object ?
            o = c.getConstructor(new Class[] {String.class}).newInstance(new String[] {(String)value});
         }
         return o;         
      }
      
      /**
       * Returns the value to which this scope maps the specified key. If the
       * scope doesn't contain a variable with the specifed name
       */
      public java.lang.Object getObject(java.lang.Object key)
      {
         java.lang.Object o = null;

         Scope s = this;
         while(s != null && !s.local.containsKey(key))
         {
            s = s.parent;
         }
         
         if(s == null)
         {
            if(request != null && key instanceof String)
            {
               String skey = (String)key;
               o = request.getParameter(skey);
               
               if(o == null)
               {
                  o = request.getSession(true).getValue(skey);
               }
            }
         }
         else
         {
            o = s.local.get(key);
         }
         
         return o;
      }

      /**
       * Removes the variables with the specifed name from the scope (or parent scope)
       */
      public void remove(Object key)
      {
         if (local.containsKey(key))
         {
            local.remove(key);
         }
         else if (parent != null)
         {
            parent.remove(key);
         }
         else if(request != null && key instanceof String)
         {
            request.getSession(true).removeValue((String)key);
         }
      }     
}
