/**
 * Copyright (C) 2000  Johan Redestig, Anders Palsson
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.codemode.bishop;

import java.util.HashMap;

import org.webmacro.Context;
import org.webmacro.ContextException;
import org.webmacro.WM;
import org.webmacro.Template;
import org.webmacro.engine.WMTemplate;
import org.webmacro.InitException;
import org.webmacro.NotFoundException;

import org.log4j.Category;

/**
 * 
 */
public class View extends HashMap
{
      protected String name;
      protected String rootTemplate;
      protected HashMap subviews;
      Category cat = Category.getInstance(View.class.getName());
      
      /**
       *
       */
      public View()
      {
         super();
         name = "";
         rootTemplate = "";
         subviews = new HashMap();
      }

      /**
       * Returns this view's name
       */
      public String getName()
      {
         return name;
      }

      /**
       * Set the name of this view.
       */
      public void setName(String name)
      {
         this.name = name;
      }

      /**
       * Sets this view's root template and removes all subviews.
       */
      public void setRootTemplate(String template)
      {
         rootTemplate = template;
         subviews.clear();
      }

      /**
       * Returns this view's root template.
       */
      public String getRootTemplate()
      {
         return rootTemplate;
      }      
      
      /**
       * Adds a subview with the specified name and template to this view.
       */
      public void addSubView(String name, String template)
      {
         subviews.put(name, template);
      }

      /**
       * Evaluates the view and returns the evaluated view (as html/wml).
       */
      public String evaluate()
      {
         String result = "";
         try
         {
            WM wm = new WM();
            HashMap currentContext = (HashMap)this.clone();
            Template root = wm.getTemplate(rootTemplate);
            Context context = wm.getContext();
            currentContext.putAll(subviews);
            context.setBean(currentContext);
            result = (String)root.evaluate(context);
         }
         catch (Throwable e)
         {
            cat.error("Evaulationerror", e);
         }         
         return result;
      }      
}
