/**
 * Copyright (C) 2000  Johan Redestig, Anders Palsson
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.codemode.bishop.entity;

import org.codemode.bishop.BishopEntity;
import org.codemode.bishop.Model;
import org.codemode.bishop.Scope;
import org.codemode.bishop.BishopException;

/**
 *
 */
public class In extends org.codemode.bishop.entity.Object 
{
      protected static final String DEFAULT_VALUE = "default-value";
      
      public void invoke(Scope locals, org.codemode.bishop.View view,  Model model) throws Throwable
      {
         // will never be invoked
      }

      public java.lang.String getDefaultValue()
      {
         return (java.lang.String)attributes.get(DEFAULT_VALUE);
      }
      

      /**
       * Returns a java.lang.Object that represents the value for thie In (Object)
       */
      public java.lang.Object getObject(java.lang.Object obj) throws BishopException
      {
         java.lang.Object o = null;
         java.lang.Object value = obj == null ? attributes.get(DEFAULT_VALUE) : obj;
         String type = getType();

         if(value == null)
            throw new BishopException(null, this, "No object or default-value");
         
         if(type == null || type.equals(value.getClass()))
            return value;
         
         if (type.equals("int"))
            o = new Integer((String)value);
         else if (type.equals("boolean"))
            o = new Boolean((String)value);
         else if (type.equals("byte"))
            o = new Byte((String)value);
         else if (type.equals("char"))
            o = new Character(((String)value).charAt(0));
         else if (type.equals("double"))
            o = new Double((String)value);
         else if (type.equals("float"))
            o = new Float((String)value);
         else if (type.equals("long"))
            o = new Long((String)value);
         else if (type.equals("short"))
            o = new Short((String)value);
         else
         {
            try
            {
               Class c = Class.forName(type);
               
               if(c.isArray())
               {
//                    Class d = c.getComponentType();
//                    if(obj.getClass().isArray())
//                    {
//                       Object[] arr = (Object[])obj;
//                       for(int i = 0; i < arr.length; i++)
//                       {
                        
//                       }
//                    }
//                    else
//                    {
//                    }
               }
               else
               {               
                  // if it is not a primitive type then it must be of a class
                  // that has a String contstructor (how would be otherwise be able
                  // to marshall the String into an Object ?
                  o = c.getConstructor(new Class[] {String.class}).newInstance(new String[] {(String)value});
               }
            }
            catch (Exception e)
            {
               throw new BishopException(e, this, "No such class for In");
            }
         }
         return o;
      }
}
