/**
 * Copyright (C) 2000  Johan Redestig, Anders Palsson
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.codemode.bishop.entity;

import org.codemode.bishop.BishopEntity;
import org.codemode.bishop.Model;
import org.codemode.bishop.Scope;
import org.codemode.bishop.BishopException;

import java.lang.reflect.InvocationTargetException;
import java.util.Vector;
import java.lang.reflect.Method;

/**
 *
 */
public class Invoke extends BishopEntity
{
      protected final static String OBJECT = "object";
      protected final static String METHOD = "method";

      protected Vector ins = new Vector();
      protected Out out = new Out();

      /**
       *
       */
      public void invoke(Scope locals, org.codemode.bishop.View view, Model model) throws Throwable
      {
         String _object = (String)attributes.get(OBJECT);
         String _method = (String)attributes.get(METHOD);
         java.lang.Object _target = null;

         _target = locals.getObject(_object);
         if(_target == null)
            throw new BishopException(null, this, "No such object: " + _object);
         
         try
         {
            Class _class = _target.getClass();
            Class[] argtypes = new Class[ins.size()];
            java.lang.Object[] args = new java.lang.Object[ins.size()];
            for(int i = 0; i < ins.size(); i++)
            {
               In in = (In)ins.get(i);
               java.lang.Object obj = locals.getObject(in);
               argtypes[i] = obj.getClass();
               args[i] = obj;
            }
            
            Method method = _class.getMethod(_method, argtypes);
            java.lang.Object result = method.invoke(_target, args);
            
            if (result != null && !(result instanceof Void))
            {
               store(result, locals, view, model);
            }
         }
         catch (InvocationTargetException ite)
         {
            throw new BishopException(ite.getTargetException(), this, "Failed to invoke");
         }
      }
            
      /**
       *
       */
      protected void store(java.lang.Object result, Scope locals, org.codemode.bishop.View view, Model model)
      {
         String _object = out.getName();
         
         if (out.isExported())
         {
            view.put(_object, result);
         }

         if (out.isStatic())
         {
            model.setObject(_object, result);
         }

         locals.putObject(_object, result);
      }

      /**
       *
       */
      public void addChild(BishopEntity child)
      {
         if(child instanceof In)
         {
            ins.add((In)child);
         }
         else if(child instanceof Out)
         {
            out = (Out)child;
         }
      }
}
