/**
 * Copyright (C) 2000  Johan Redestig, Anders Palsson
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.codemode.bishop.entity;

import org.codemode.bishop.BishopEntity;
import org.codemode.bishop.Model;
import org.codemode.bishop.Scope;
import org.codemode.bishop.View;
import org.codemode.bishop.BishopException;

import java.text.MessageFormat;
import java.util.Vector;

import org.log4j.Category;

public class Trace extends BishopEntity
{
      protected final static String MSG = "msg";
      protected final static String TYPE = "type";      
      protected Vector ins = new Vector();

      Category cat = Category.getInstance(Trace.class.getName());
      
      /**
       *
       */
      public void invoke(Scope locals, View view, Model model) throws Throwable
      {
         String msg = getMsg();
         try
         {
            java.lang.Object o = locals.getObject(msg);            
            if (o == null)
            {
               java.lang.Object[] args = new java.lang.Object[ins.size()];
               for(int i = 0; i < args.length; ++i)
                  args[i] = locals.getObject((In)ins.get(i));
               msg = MessageFormat.format(msg, args);
            }
            else
            {
               msg = o.toString();
            }

            String type = (String)attributes.get(TYPE);
            if(type.equals("info"))
               cat.info(msg);
            else if(type.equals("warn"))
               cat.warn(msg);
            else if(type.equals("error"))
               cat.error(msg);
            else
               cat.debug(msg);
         }
         catch (Exception e)
         {
            throw new BishopException(e, this, "Failed to perform trace");
         }
      }

      public String getMsg()
      {
         return (String)attributes.get(MSG);
      }


      public void addChild(BishopEntity child)
      {
         if(child instanceof In)
         {
            ins.add(child);
         }
      }
}

