/**
 * Copyright (C) 2000  Johan Redestig, Anders Palsson
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.codemode.bishop.entity;

import org.codemode.bishop.BishopEntity;
import org.codemode.bishop.Model;
import org.codemode.bishop.Scope;
import org.codemode.bishop.BishopException;

public class If extends BishopEntity
{
      protected static final String NAME = "name";
      protected static final String VALUE = "value";

      protected Else elseStatement;

      /**
       *
       */
      public void invoke(Scope locals, org.codemode.bishop.View view, Model model) throws Throwable
      {
         String name = (String)attributes.get(NAME);
         String value = (String)attributes.get(VALUE);
         boolean result = false;

         if (value == null)
         {
            // check if the variable is defined
            result = locals.getObject(name) != null;
         }
         else
         {
            java.lang.Object object = locals.getObject(name);
            if (object != null)
            {
               Class c = object.getClass();
               java.lang.Object par = c.getConstructor(new Class[] {String.class}).newInstance(new String[] {(String)value});
               result = par.equals(object);
            }
            else
            {
               throw new BishopException(null, this, "No such object: " + name);
            }
         }

         if(result)
         {
            invokeAllChildren(locals, view, model);
         }
         else
         {
            if (elseStatement != null)
            {
               elseStatement.invoke(locals, view, model);
            }
         }            
      }

      /**
       *
       */
      public void addChild(BishopEntity child)
      {
         if (child instanceof Else)
         {
            elseStatement = (Else)child;
         }
         else
         {
            super.addChild(child);
         }
      }
}
